//
//  TheDotView.m
//  TheDot
//
//  Created by Alexander Thomas on Mon Jul 29 2002.
//  doctor.lex@gmail.com
//  http://go.to/dr.lex
//  Copyright (c) 2002-2009. All rights reserved.
//

#import "TheDotView.h"

// Shows the value of 'val' as a string in the IBOutlet id 'field', in the form "12.3".
void showSliderVal( id field, float val )
{
	char str[8];
	int x = val;
	int y = (val-x)*10;
	// There's probably a cleaner way to do this but I'm too much of an ObjC noob for that.
	sprintf(str,"%d.%d",x,y);
	[field setStringValue:[NSString stringWithCString:str encoding:NSUTF8StringEncoding]];
}


@implementation TheDotView

// Initialize screensaver
- (id) initWithFrame: (NSRect)frame isPreview: (BOOL)isPreview
{
	self = [super initWithFrame:frame isPreview:isPreview];
	if(! self)
		return nil;
	else {
		// load user defaults
		defaults = [[ScreenSaverDefaults defaultsForModuleWithName:@"com.drlex.thedotSaver1"] retain];
		
		if (![defaults boolForKey:@"defaultsSet"]) // Not yet initialized
			[self loadFactorySettings];
		else {
			NSScanner *theColor;
			NSMutableCharacterSet *skipIt;
			float r,g,b;
			diameter = [defaults floatForKey:@"radius"];
			speed = [defaults floatForKey:@"speed"];
			step = [defaults floatForKey:@"step"];
			shape = [defaults integerForKey:@"shape"];
			// All the following 8 lines of code do, is retrieving the color.
			// Simply attempting to store the color object results in a nice crash.
			// If you know a simpler way to achieve this, please mail me at doctor.lex@gmail.com !
			theColor = [NSScanner scannerWithString:[defaults objectForKey:@"color"]];
			// The stored string consists of a word followed by 4 float values separated by spaces.
			// We use setCharactersToBeSkipped to skip the word and the spaces.
			skipIt = [NSMutableCharacterSet letterCharacterSet]; // causes "incompatible pointer type", but it works
			[skipIt formUnionWithCharacterSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
			[theColor setCharactersToBeSkipped:skipIt];
			[theColor scanFloat:&r];
			[theColor scanFloat:&g];
			[theColor scanFloat:&b];
			color = [[NSColor colorWithCalibratedRed:r green:g blue:b alpha:1.0] retain];
		}
		
		center.x = 0.5;
		center.y = frame.size.height-1.5;
		[self setAnimationTimeInterval:1/speed];
	}
	
	return self;
}

- (void) loadFactorySettings
{
	diameter = 2.0;
	speed = 50.0;
	step = 2.0;
	shape = 0;
	color = [[NSColor greenColor] retain];
}  

// Update and store the user defaults
- (void) writeDefaults
{
	[defaults setBool:YES forKey:@"defaultsSet"];
	[defaults setFloat:diameter forKey:@"radius"];
	[defaults setFloat:speed forKey:@"speed"];
	[defaults setFloat:step forKey:@"step"];
	[defaults setInteger:shape forKey:@"shape"];
	[defaults setObject:[color description] forKey:@"color"]; // without 'description': crash!!!
	
	[defaults synchronize];
}

// This could be used if the initialisation of the screensaver involves lengthy procedures
// and the animation can only start after these have been completed. Of course this is not
// the case with this screensaver!
- (void)startAnimation
{
    [super startAnimation];
}

- (void)stopAnimation
{
    [super stopAnimation];
}

// Draw the "background" for the screensaver, in this case the obvious black screen.
- (void) drawRect: (NSRect)rect
{
	// Paint the screen black.
	[[NSColor blackColor] set];
	NSRectFill([self bounds]);   // Equiv to [[NSBezierPath bezierPathWithRect:[self bounds]] fill]
}

// This is executed every 1/speed seconds, as set with setAnimationTimeInterval.
- (void) animateOneFrame
{
	NSRect screenRect = [self bounds];
	
	center.x += step;
	if( center.x >= screenRect.size.width ) {
		center.x = 0.5;
		center.y -= step;
	}
	if( center.y < 0 )
		center.y = screenRect.size.height-1.5;
	
	// Erase previous dot, expand rect to avoid antialiasing artefacts.
	dotRect.origin.x -= 1;
	dotRect.origin.y -= 1;
	dotRect.size.width += 2;
	dotRect.size.height += 2;
	
	[[NSColor blackColor] set];
	NSRectFill(dotRect);
	
	// Draw the Dot!
	dotRect.origin.x = center.x - diameter/2.0;
	dotRect.origin.y = center.y - diameter/2.0;
	dotRect.size.width  = diameter;
	dotRect.size.height = diameter;
	
	[color set];
	if( shape == 0 )
		NSRectFill(dotRect);
	else
		[[NSBezierPath bezierPathWithOvalInRect:dotRect] fill];
}

// Indicates that there is a config sheet.
- (BOOL) hasConfigureSheet
{
	return YES;
}

// Creates the config sheet and returns a pointer to it.
- (NSWindow*) configureSheet
{
	// Create the sheet if it hasn't been created yet.
	if (configureSheet == nil)
		[NSBundle loadNibNamed:@"TheDot" owner:self];
	
	// Load the user defaults into the fields and sliders.
	[self loadValues];
	
	return configureSheet;
}

- (void) loadValues
{
	[IBradius setFloatValue:diameter];
	showSliderVal(IBradiusView,diameter);
	[IBspeed setFloatValue:speed];
	showSliderVal(IBspeedView,speed);
	[IBstep setFloatValue:step];
	showSliderVal(IBstepView,step);
	if( shape == 0 )
		[IBshape selectCell:IBsquare];
	else
		[IBshape selectCell:IBround];
	[IBcolor setColor:color];
}

// Close the config sheet and store the user defaults.
- (IBAction) closeSheet: (id)sender
{
	[self writeDefaults];
	[NSApp endSheet:configureSheet];
}

// The following are messages sent when the user interacts with the controls of the config sheet.
// Because the sliders are set to "continuous", they will keep sending their messages as long as
// the user is dragging them. This is necessary for the real-time displaying of the exact values.
- (IBAction) setRadius: (id)sender
{
	diameter = [sender floatValue];
	showSliderVal(IBradiusView,diameter);
}

- (IBAction) setSpeed: (id)sender
{
	speed = [sender floatValue];
	showSliderVal(IBspeedView,speed);
	[self setAnimationTimeInterval:1/speed];
}

- (IBAction) setStep: (id)sender
{
	step = [sender floatValue];
	showSliderVal(IBstepView,step);
}

- (IBAction) setShape: (id)sender
{
	shape = [sender selectedTag];
}

- (IBAction) setColor: (id)sender
{
	[color autorelease];
	color = [[sender color] retain];
}

@end
